@@ -35,9 +35,20 @@ inline
 void
 Base<elem_type,derived>::print(const std::string extra_text) const
   {
+  arma_extra_debug_sigprint();
+  
   const quasi_unwrap<derived> tmp( (*this).get_ref() );
   
-  tmp.M.impl_print(extra_text);
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = get_cout_stream().width();
+    
+    get_cout_stream() << extra_text << '\n';
+    
+    get_cout_stream().width(orig_width);
+    }
+  
+  arma_ostream::print(get_cout_stream(), tmp.M, true);
   }
 
 
@@ -48,9 +59,20 @@ inline
 void
 Base<elem_type,derived>::print(std::ostream& user_stream, const std::string extra_text) const
   {
+  arma_extra_debug_sigprint();
+  
   const quasi_unwrap<derived> tmp( (*this).get_ref() );
   
-  tmp.M.impl_print(user_stream, extra_text);
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = user_stream.width();
+    
+    user_stream << extra_text << '\n';
+    
+    user_stream.width(orig_width);
+    }
+  
+  arma_ostream::print(user_stream, tmp.M, true);
   }
   
 
@@ -61,9 +83,20 @@ inline
 void
 Base<elem_type,derived>::raw_print(const std::string extra_text) const
   {
+  arma_extra_debug_sigprint();
+  
   const quasi_unwrap<derived> tmp( (*this).get_ref() );
   
-  tmp.M.impl_raw_print(extra_text);
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = get_cout_stream().width();
+    
+    get_cout_stream() << extra_text << '\n';
+    
+    get_cout_stream().width(orig_width);
+    }
+  
+  arma_ostream::print(get_cout_stream(), tmp.M, false);
   }
 
 
@@ -74,9 +107,68 @@ inline
 void
 Base<elem_type,derived>::raw_print(std::ostream& user_stream, const std::string extra_text) const
   {
+  arma_extra_debug_sigprint();
+  
   const quasi_unwrap<derived> tmp( (*this).get_ref() );
   
-  tmp.M.impl_raw_print(user_stream, extra_text);
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = user_stream.width();
+    
+    user_stream << extra_text << '\n';
+    
+    user_stream.width(orig_width);
+    }
+  
+  arma_ostream::print(user_stream, tmp.M, false);
+  }
+
+
+
+template<typename elem_type, typename derived>
+arma_cold
+inline
+void
+Base<elem_type,derived>::brief_print(const std::string extra_text) const
+  {
+  arma_extra_debug_sigprint();
+  
+  const quasi_unwrap<derived> tmp( (*this).get_ref() );
+  
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = get_cout_stream().width();
+    
+    get_cout_stream() << extra_text << '\n';
+    
+    get_cout_stream().width(orig_width);
+    }
+  
+  arma_ostream::brief_print(get_cout_stream(), tmp.M);
+  }
+
+
+
+template<typename elem_type, typename derived>
+arma_cold
+inline
+void
+Base<elem_type,derived>::brief_print(std::ostream& user_stream, const std::string extra_text) const
+  {
+  arma_extra_debug_sigprint();
+  
+  const quasi_unwrap<derived> tmp( (*this).get_ref() );
+  
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = user_stream.width();
+    
+    user_stream << extra_text << '\n';
+    
+    user_stream.width(orig_width);
+    }
+  
+  arma_ostream::brief_print(user_stream, tmp.M);
   }
 
 
@@ -375,6 +467,59 @@ Base<elem_type,derived>::is_hermitian(const typename get_pod_type<elem_type>::re
 
 
 
+template<typename elem_type, typename derived>
+inline
+arma_warn_unused
+bool
+Base<elem_type,derived>::is_zero(const typename get_pod_type<elem_type>::result tol) const
+  {
+  arma_extra_debug_sigprint();
+  
+  typedef typename get_pod_type<elem_type>::result T;
+  
+  arma_debug_check( (tol < T(0)), "is_zero(): parameter 'tol' must be >= 0" );
+  
+  if(Proxy<derived>::use_at || is_Mat<typename Proxy<derived>::stored_type>::value)
+    {
+    const quasi_unwrap<derived> U( (*this).get_ref() );
+    
+    return arrayops::is_zero( U.M.memptr(), U.M.n_elem, tol );
+    }
+  
+  const Proxy<derived> P( (*this).get_ref() );
+  
+  const uword n_elem = P.get_n_elem();
+  
+  if(n_elem == 0)  { return false; }
+  
+  const typename Proxy<derived>::ea_type Pea = P.get_ea();
+  
+  if(is_cx<elem_type>::yes)
+    {
+    for(uword i=0; i<n_elem; ++i)
+      {
+      const elem_type val = Pea[i];
+      
+      const T val_real = access::tmp_real(val);
+      const T val_imag = access::tmp_imag(val);
+      
+      if(eop_aux::arma_abs(val_real) > tol)  { return false; }
+      if(eop_aux::arma_abs(val_imag) > tol)  { return false; }
+      }
+    }
+  else  // not complex
+    {
+    for(uword i=0; i<n_elem; ++i)
+      {
+      if(eop_aux::arma_abs(Pea[i]) > tol)  { return false; }
+      }
+    }
+  
+  return true;
+  }
+
+
+
 template<typename elem_type, typename derived>
 inline
 arma_warn_unused
@@ -427,19 +572,25 @@ Base<elem_type,derived>::is_diagmat() const
   
   if(A.n_elem <= 1)  { return true; }
   
+  // NOTE: we're NOT assuming the matrix has a square size
+  
   const uword A_n_rows = A.n_rows;
   const uword A_n_cols = A.n_cols;
   
-  const elem_type* A_colmem = A.memptr();
+  const elem_type* A_mem = A.memptr();
+  
+  if(A_mem[1] != elem_type(0))  { return false; }
+  
+  // if we got to this point, do a thorough check
   
   for(uword A_col=0; A_col < A_n_cols; ++A_col)
     {
     for(uword A_row=0; A_row < A_n_rows; ++A_row)
       {
-      if( (A_colmem[A_row] != elem_type(0)) && (A_row != A_col) )  { return false; }
+      if( (A_mem[A_row] != elem_type(0)) && (A_row != A_col) )  { return false; }
       }
     
-    A_colmem += A_n_rows;
+    A_mem += A_n_rows;
     }
   
   return true;
@@ -664,7 +815,8 @@ Base<elem_type,derived>::has_nan() const
 
 
 template<typename elem_type, typename derived>
-arma_inline
+inline
+arma_warn_unused
 const Op<derived,op_vectorise_col>
 Base<elem_type, derived>::as_col() const
   {
@@ -674,7 +826,8 @@ Base<elem_type, derived>::as_col() const
 
 
 template<typename elem_type, typename derived>
-arma_inline
+inline
+arma_warn_unused
 const Op<derived,op_vectorise_row>
 Base<elem_type, derived>::as_row() const
   {
@@ -687,7 +840,8 @@ Base<elem_type, derived>::as_row() const
 // extra functions defined in Base_extra_yes
 
 template<typename elem_type, typename derived>
-arma_inline
+inline
+arma_warn_unused
 const Op<derived,op_inv>
 Base_extra_yes<elem_type, derived>::i() const
   {
@@ -696,32 +850,6 @@ Base_extra_yes<elem_type, derived>::i() const
 
 
 
-template<typename elem_type, typename derived>
-arma_deprecated
-inline
-const Op<derived,op_inv>
-Base_extra_yes<elem_type, derived>::i(const bool) const   // argument kept only for compatibility with old user code
-  {
-  // arma_debug_warn(".i(bool) is deprecated and will be removed; change to .i()");
-  
-  return Op<derived,op_inv>(static_cast<const derived&>(*this));
-  }
-
-
-
-template<typename elem_type, typename derived>
-arma_deprecated
-inline
-const Op<derived,op_inv>
-Base_extra_yes<elem_type, derived>::i(const char*) const   // argument kept only for compatibility with old user code
-  {
-  // arma_debug_warn(".i(char*) is deprecated and will be removed; change to .i()");
-  
-  return Op<derived,op_inv>(static_cast<const derived&>(*this));
-  }
-
-
-
 template<typename elem_type, typename derived>
 inline
 arma_warn_unused
